movie<-read.csv('/Users/Thierry/Documents/R/Data/Economics/Greene/movie_buzz.csv',
                header=TRUE,sep=",")

# Preprocessing

# declaring the factors
movie$MPRATING<-as.factor(movie$MPRATING)
levels(movie$MPRATING)<-c("G","PG","PG13","R")
movie$SEQUEL<-as.factor(movie$SEQUEL)
movie$ACTION<-as.factor(movie$ACTION)
movie$COMEDY<-as.factor(movie$COMEDY)
movie$ANIMATED<-as.factor(movie$ANIMATED)
movie$HORROR<-as.factor(movie$HORROR)
# Log transformation
movie$BOX<-log(movie$BOX)
movie$BUDGET<-log(movie$BUDGET)

X<-model.matrix(BOX~.,data=movie)

# Q1
n<-nrow(movie)
ntrain<-40
ntest<-n-ntrain
train<-sample(n,ntrain)
movie.train<-movie[train,]
movie.test<-movie[-train,]
X.train<-X[train,]
X.test<-X[-train,]

# Q2
library("leaps")
# Forward selection 
reg.forward<-regsubsets(BOX~.,data=movie.train,method='forward',
                        nvmax=15)
plot(reg.forward,scale="bic")
plot(reg.forward,scale="adjr2")
res<-summary(reg.forward)

best.bic<-which.min(res$bic)
coef(reg.forward,best.bic)

ypred<-X.test[,res$which[best.bic,]]%*%coef(reg.forward,best.bic)
mse_forward_bic<-mean((ypred-movie.test$BOX)^2)
plot(movie.test$BOX,ypred)
abline(0,1)

best.adjr2<-which.max(res$adjr2)
coef(reg.forward,best.adjr2)

ypred<-X.test[,res$which[best.adjr2,]]%*%coef(reg.forward,best.adjr2)
mse_forward_adjr2<-mean((ypred-movie.test$BOX)^2)

print(c(mse_forward_bic,mse_forward_adjr2))

# Backward selection 
reg.backward<-regsubsets(BOX~.,data=movie.train,method='backward')
plot(reg.backward,scale="bic") # same as forward
plot(reg.backward,scale="adjr2") # this one is different

res<-summary(reg.backward)

best.adjr2<-which.max(res$adjr2)
coef(reg.backward,best.adjr2)

ypred<-X.test[,res$which[best.adjr2,]]%*%coef(reg.backward,best.adjr2)
mse_backward_adjr2<-mean((ypred-movie.test$BOX)^2)

print(c(mse_forward_bic,mse_forward_adjr2,mse_backward_adjr2))


# Optimal subset

reg.exhaustive<-regsubsets(BOX~.,data=movie.train,method='exhaustive',
                           nvmax=15)
plot(reg.exhaustive,scale="bic") # same as before
plot(reg.exhaustive,scale="adjr2")


# Ridge
library(glmnet)

cv.out<-cv.glmnet(X.train[,-1],movie.train$BOX,alpha=0,standardize=TRUE)
plot(cv.out)
fit1<-glmnet(X.train[,-1],movie.train$BOX,lambda=cv.out$lambda.min,
             alpha=0,standardize=TRUE)
fit2<-glmnet(X.train[,-1],movie.train$BOX,lambda=cv.out$lambda.1se,
             alpha=0,standardize=TRUE)
# Minimum CV error model
ridge.pred1<-predict(fit1,newx=X.test[,-1])
mse_ridge1<-mean((movie.test$BOX-ridge.pred1)^2)
# One SE rule
ridge.pred2<-predict(fit2,newx=X.test[,-1])
mse_ridge2<-mean((movie.test$BOX-ridge.pred2)^2)
print(c(mse_forward_bic,mse_forward_adjr2,mse_backward_adjr2,
        mse_ridge1,mse_ridge2))

# lasso
cv.out<-cv.glmnet(X.train[,-1],movie.train$BOX,alpha=1,standardize=TRUE)
plot(cv.out)
fit1<-glmnet(X.train[,-1],movie.train$BOX,lambda=cv.out$lambda.min,
             alpha=1,standardize=TRUE)
fit2<-glmnet(X.train[,-1],movie.train$BOX,lambda=cv.out$lambda.1se,
             alpha=1,standardize=TRUE)

lasso.pred1<-predict(fit1,newx=X.test[,-1])
mse_lasso1<-mean((movie.test$BOX-lasso.pred1)^2)
lasso.pred2<-predict(fit2,newx=X.test[,-1])
mse_lasso2<-mean((movie.test$BOX-lasso.pred2)^2)

print(c(mse_forward_bic,mse_forward_adjr2,mse_backward_adjr2,
        mse_ridge1,mse_ridge2, mse_lasso1,mse_lasso2))


